@@ -8,12 +8,13 @@ from datetime import datetime, timedelta
 from itertools import chain
 
 import requests
-from requests import HTTPError
 from urlobject import URLObject
 import six
 from six.moves.urllib.parse import urlencode
 from nylas._client_sdk_version import __VERSION__
+from nylas.client.delta_collection import DeltaCollection
 from nylas.client.errors import MessageRejectedError, NylasApiError
+from nylas.client.outbox_models import Outbox
 from nylas.client.restful_model_collection import RestfulModelCollection
 from nylas.client.restful_models import (
     Calendar,
@@ -31,16 +32,18 @@ from nylas.client.restful_models import (
     Draft,
     Component,
     JobStatus,
+    Webhook,
 )
 from nylas.client.neural_api_models import Neural
 from nylas.client.scheduler_restful_model_collection import (
     SchedulerRestfulModelCollection,
 )
-from nylas.utils import timestamp_from_dt, create_request_body
+from nylas.client.authentication_models import Authentication
+from nylas.utils import timestamp_from_dt, create_request_body, AuthMethod, HttpMethod
 
 DEBUG = environ.get("NYLAS_CLIENT_DEBUG")
 API_SERVER = "https://api.nylas.com"
-SUPPORTED_API_VERSION = "2.2"
+SUPPORTED_API_VERSION = "2.5"
 
 
 def _validate(response):
@@ -119,17 +122,13 @@ class APIClient(json.JSONEncoder):
         self.admin_session = requests.Session()
 
         if client_secret is not None:
-            b64_client_secret = b64encode((client_secret + ":").encode("utf8"))
-            authorization = "Basic {secret}".format(
-                secret=b64_client_secret.decode("utf8")
-            )
             self.admin_session.headers = {
-                "Authorization": authorization,
                 "X-Nylas-API-Wrapper": "python",
                 "X-Nylas-Client-Id": self.client_id,
                 "Nylas-API-Version": self.api_version,
                 "User-Agent": version_header,
             }
+            self.admin_session.headers.update(self._add_auth_header(AuthMethod.BASIC))
         super(APIClient, self).__init__()
 
     @property
@@ -139,12 +138,6 @@ class APIClient(json.JSONEncoder):
     @access_token.setter
     def access_token(self, value):
         self._access_token = value
-        if value:
-            authorization = "Bearer {token}".format(token=value)
-            self.session.headers["Authorization"] = authorization
-        else:
-            if "Authorization" in self.session.headers:
-                del self.session.headers["Authorization"]
 
     def authentication_url(
         self,
@@ -191,8 +184,11 @@ class APIClient(json.JSONEncoder):
             "Accept": "application/json",
         }
 
-        resp = requests.post(
-            self.access_token_url, data=urlencode(args), headers=headers
+        resp = self._request(
+            HttpMethod.POST,
+            self.access_token_url,
+            headers=headers,
+            data=urlencode(args),
         )
         results = _validate(resp).json()
 
@@ -266,9 +262,10 @@ class APIClient(json.JSONEncoder):
         token_info_url = self.token_info_url.format(
             client_id=self.client_id, account_id=self.account.id
         )
-        self.admin_session.headers["Content-Type"] = "application/json"
+        headers = {"Content-Type": "application/json"}
+        headers.update(self.admin_session.headers)
         resp = self.admin_session.post(
-            token_info_url, json={"access_token": self.access_token}
+            token_info_url, headers=headers, json={"access_token": self.access_token}
         )
         _validate(resp).json()
         return resp.json()
@@ -290,7 +287,7 @@ class APIClient(json.JSONEncoder):
             "start_time": start_time,
             "end_time": end_time,
         }
-        resp = self.session.post(url, json=data)
+        resp = self._request(HttpMethod.POST, url, json=data)
         _validate(resp)
         return resp.json()
 
@@ -352,12 +349,15 @@ class APIClient(json.JSONEncoder):
             "interval_minutes": interval_minutes,
             "start_time": start_time,
             "end_time": end_time,
-            "buffer": buffer,
-            "round_robin": round_robin,
             "free_busy": free_busy or [],
             "open_hours": open_hours or [],
         }
-        resp = self.session.post(url, json=data)
+        if buffer is not None:
+            data["buffer"] = buffer
+        if round_robin is not None:
+            data["round_robin"] = round_robin
+
+        resp = self._request(HttpMethod.POST, url, json=data)
         _validate(resp)
         return resp.json()
 
@@ -404,11 +404,13 @@ class APIClient(json.JSONEncoder):
             "interval_minutes": interval_minutes,
             "start_time": start_time,
             "end_time": end_time,
-            "buffer": buffer,
             "free_busy": free_busy or [],
             "open_hours": open_hours or [],
         }
-        resp = self.session.post(url, json=data)
+        if buffer is not None:
+            data["buffer"] = buffer
+
+        resp = self._request(HttpMethod.POST, url, json=data)
         _validate(resp)
         return resp.json()
 
@@ -474,10 +476,26 @@ class APIClient(json.JSONEncoder):
     def components(self):
         return RestfulModelCollection(Component, self)
 
+    @property
+    def deltas(self):
+        return DeltaCollection(self)
+
+    @property
+    def webhooks(self):
+        return RestfulModelCollection(Webhook, self)
+
     @property
     def neural(self):
         return Neural(self)
 
+    @property
+    def outbox(self):
+        return Outbox(self)
+
+    @property
+    def authentication(self):
+        return Authentication(self)
+
     ##########################################################
     #   Private functions used by Restful Model Collection   #
     ##########################################################
@@ -510,22 +528,30 @@ class APIClient(json.JSONEncoder):
 
         converted_data = create_request_body(filters, cls.datetime_filter_attrs)
         url = str(URLObject(url).add_query_params(converted_data.items()))
-        response = self._get_http_session(cls.api_root).get(url)
+        response = self._request(HttpMethod.GET, url, cls=cls)
         results = _validate(response).json()
         return [cls.create(self, **x) for x in results if x is not None]
 
     def _get_resource_raw(
-        self, cls, id, extra=None, headers=None, stream=False, path=None, **filters
+        self,
+        cls,
+        resource_id,
+        extra=None,
+        headers=None,
+        stream=False,
+        path=None,
+        stream_timeout=None,
+        **filters
     ):
         """Get an individual REST resource"""
         if path is None:
             path = cls.collection_name
         postfix = "/{}".format(extra) if extra else ""
         path = "/{}".format(path) if path else ""
-        id = "/{}".format(id) if id else ""
+        resource_id = "/{}".format(resource_id) if resource_id else ""
         if not cls.api_root:
             url = "{server}{path}{id}{postfix}".format(
-                server=self.api_server, path=path, id=id, postfix=postfix
+                server=self.api_server, path=path, id=resource_id, postfix=postfix
             )
         else:
             url = "{server}/{prefix}/{client_id}{path}{id}{postfix}".format(
@@ -533,30 +559,33 @@ class APIClient(json.JSONEncoder):
                 prefix=cls.api_root,
                 client_id=self.client_id,
                 path=path,
-                id=id,
+                id=resource_id,
                 postfix=postfix,
             )
 
         converted_data = create_request_body(filters, cls.datetime_filter_attrs)
         url = str(URLObject(url).add_query_params(converted_data.items()))
 
-        session = self._get_http_session(cls.api_root)
-
-        headers = headers or {}
-        headers.update(session.headers)
-        response = session.get(url, headers=headers, stream=stream)
+        response = self._request(
+            HttpMethod.GET,
+            url,
+            cls=cls,
+            headers=headers,
+            stream=stream,
+            timeout=stream_timeout,
+        )
         return _validate(response)
 
-    def _get_resource(self, cls, id, **filters):
-        response = self._get_resource_raw(cls, id, **filters)
+    def _get_resource(self, cls, resource_id, **filters):
+        response = self._get_resource_raw(cls, resource_id, **filters)
         result = response.json()
         if isinstance(result, list):
             result = result[0]
         return cls.create(self, **result)
 
-    def _get_resource_data(self, cls, id, extra=None, headers=None, **filters):
+    def _get_resource_data(self, cls, resource_id, extra=None, headers=None, **filters):
         response = self._get_resource_raw(
-            cls, id, extra=extra, headers=headers, **filters
+            cls, resource_id, extra=extra, headers=headers, **filters
         )
         return response.content
 
@@ -573,15 +602,14 @@ class APIClient(json.JSONEncoder):
             .set_query_params(**kwargs)
         )
 
-        session = self._get_http_session(cls.api_root)
-
         if cls == File:
-            response = session.post(url, files=data)
+            response = self._request(HttpMethod.POST, url, cls=cls, files=data)
         else:
             converted_data = create_request_body(data, cls.datetime_attrs)
             headers = {"Content-Type": "application/json"}
-            headers.update(session.headers)
-            response = session.post(url, json=converted_data, headers=headers)
+            response = self._request(
+                HttpMethod.POST, url, cls=cls, headers=headers, json=converted_data
+            )
 
         result = _validate(response).json()
         if cls.collection_name == "send":
@@ -596,22 +624,22 @@ class APIClient(json.JSONEncoder):
             path="/{}".format(cls.collection_name) if cls.collection_name else "",
         )
         url = URLObject(self.api_server).with_path("{name}".format(name=name))
-        session = self._get_http_session(cls.api_root)
 
         if cls == File:
-            response = session.post(url, files=data)
+            response = self._request(HttpMethod.POST, url, cls=cls, files=data)
         else:
             converted_data = [
                 create_request_body(datum, cls.datetime_attrs) for datum in data
             ]
             headers = {"Content-Type": "application/json"}
-            headers.update(session.headers)
-            response = session.post(url, json=converted_data, headers=headers)
+            response = self._request(
+                HttpMethod.POST, url, cls=cls, headers=headers, json=converted_data
+            )
 
         results = _validate(response).json()
         return [cls.create(self, **x) for x in results]
 
-    def _delete_resource(self, cls, id, data=None, **kwargs):
+    def _delete_resource(self, cls, resource_id, data=None, **kwargs):
         name = "{prefix}{path}".format(
             prefix="/{}/{}".format(cls.api_root, self.client_id)
             if cls.api_root
@@ -620,16 +648,17 @@ class APIClient(json.JSONEncoder):
         )
         url = (
             URLObject(self.api_server)
-            .with_path("{name}/{id}".format(name=name, id=id))
+            .with_path("{name}/{id}".format(name=name, id=resource_id))
             .set_query_params(**kwargs)
         )
-        session = self._get_http_session(cls.api_root)
         if data:
-            _validate(session.delete(url, json=data))
+            _validate(self._request(HttpMethod.DELETE, url, cls=cls, json=data))
         else:
-            _validate(session.delete(url))
+            _validate(self._request(HttpMethod.DELETE, url, cls=cls))
 
-    def _put_resource(self, cls, id, data, extra=None, path=None, **kwargs):
+    def _request_update_resource(
+        self, method, cls, resource_id, data, extra=None, path=None, **kwargs
+    ):
         if path is None:
             path = cls.collection_name
         name = "{prefix}{path}".format(
@@ -642,64 +671,76 @@ class APIClient(json.JSONEncoder):
         postfix = "/{}".format(extra) if extra else ""
         url = (
             URLObject(self.api_server)
-            .with_path("{name}/{id}{postfix}".format(name=name, id=id, postfix=postfix))
+            .with_path(
+                "{name}/{id}{postfix}".format(
+                    name=name, id=resource_id, postfix=postfix
+                )
+            )
             .set_query_params(**kwargs)
         )
-
-        session = self._get_http_session(cls.api_root)
-
         converted_data = create_request_body(data, cls.datetime_attrs)
-        response = session.put(url, json=converted_data)
+
+        response = self._request(method, url, cls=cls, json=converted_data)
 
         result = _validate(response)
         return result.json()
 
-    def _update_resource(self, cls, id, data, **kwargs):
-        result = self._put_resource(cls, id, data, **kwargs)
+    def _patch_resource(self, cls, resource_id, data, extra=None, path=None, **kwargs):
+        return self._request_update_resource(
+            HttpMethod.PATCH, cls, resource_id, data, extra=extra, path=path, **kwargs
+        )
+
+    def _put_resource(self, cls, resource_id, data, extra=None, path=None, **kwargs):
+        return self._request_update_resource(
+            HttpMethod.PUT, cls, resource_id, data, extra=extra, path=path, **kwargs
+        )
+
+    def _update_resource(self, cls, resource_id, data, **kwargs):
+        result = self._put_resource(cls, resource_id, data, **kwargs)
         return cls.create(self, **result)
 
-    def _post_resource(self, cls, id, method_name, data, path=None):
+    def _post_resource(self, cls, resource_id, method_name, data, path=None):
         if path is None:
             path = cls.collection_name
         path = "/{}".format(path) if path else ""
-        id = "/{}".format(id) if id else ""
+        resource_id = "/{}".format(resource_id) if resource_id else ""
         method = "/{}".format(method_name) if method_name else ""
         if not cls.api_root:
-            url_path = "{name}{id}{method}".format(name=path, id=id, method=method)
+            url_path = "{name}{id}{method}".format(
+                name=path, id=resource_id, method=method
+            )
         else:
             # Management method.
             url_path = "/{prefix}/{client_id}{path}{id}{method}".format(
                 prefix=cls.api_root,
                 client_id=self.client_id,
                 path=path,
-                id=id,
+                id=resource_id,
                 method=method,
             )
 
         url = URLObject(self.api_server).with_path(url_path)
         converted_data = create_request_body(data, cls.datetime_attrs)
 
-        session = self._get_http_session(cls.api_root)
-        response = session.post(url, json=converted_data)
-
+        response = self._request(HttpMethod.POST, url, cls=cls, json=converted_data)
         return _validate(response).json()
 
-    def _call_resource_method(self, cls, id, method_name, data):
+    def _call_resource_method(self, cls, resource_id, method_name, data):
         """POST a dictionary to an API method,
         for example /a/.../accounts/id/upgrade"""
 
-        result = self._post_resource(cls, id, method_name, data)
+        result = self._post_resource(cls, resource_id, method_name, data)
         return cls.create(self, **result)
 
-    def _request_neural_resource(self, cls, data, path=None, method="PUT"):
+    def _request_neural_resource(self, cls, data, path=None, method=None):
         if path is None:
             path = cls.collection_name
+        if method is None:
+            method = HttpMethod.PUT
         url = URLObject(self.api_server).with_path("/neural/{name}".format(name=path))
 
-        session = self._get_http_session(cls.api_root)
-
         converted_data = create_request_body(data, cls.datetime_attrs)
-        response = session.request(method, url, json=converted_data)
+        response = self._request(method, url, cls=cls, json=converted_data)
 
         result = _validate(response).json()
         if isinstance(result, list):
@@ -726,3 +767,43 @@ class APIClient(json.JSONEncoder):
                 raise ValueError(
                     "Open Hours cannot contain an email not present in the main email list or the free busy email list."
                 )
+
+    def _request(self, method, url, cls=None, headers=None, **kwargs):
+        api_root = None
+        auth_method = None
+        if cls:
+            api_root = cls.api_root
+            auth_method = cls.auth_method
+
+        session = self._get_http_session(api_root)
+        headers = headers or {}
+        headers.update(session.headers)
+        headers.update(self._add_auth_header(auth_method))
+        return session.request(method.name, url, headers=headers, **kwargs)
+
+    def _add_auth_header(self, auth_method):
+        authorization = None
+        if auth_method is AuthMethod.BEARER:
+            authorization = (
+                "Bearer {token}".format(token=self.access_token)
+                if self.access_token
+                else None
+            )
+        elif auth_method is AuthMethod.BASIC_CLIENT_ID_AND_SECRET:
+            if self.client_id and self.client_secret:
+                credential = "{client_id}:{client_secret}".format(
+                    client_id=self.client_id, client_secret=self.client_secret
+                )
+                authorization = "Basic {credential}".format(
+                    credential=b64encode(credential.encode("utf8"))
+                )
+        else:
+            if self.client_secret:
+                b64_client_secret = b64encode(
+                    ("{}:".format(self.client_secret)).encode("utf8")
+                )
+                authorization = "Basic {secret}".format(
+                    secret=b64_client_secret.decode("utf8")
+                )
+
+        return {"Authorization": authorization} if authorization else {}
