@@ -1,10 +1,11 @@
 from datetime import datetime
 from collections import defaultdict
+from enum import Enum
 
 from six import StringIO
 from nylas.client.restful_model_collection import RestfulModelCollection
 from nylas.client.errors import FileUploadError, UnSyncedError, NylasApiError
-from nylas.utils import timestamp_from_dt
+from nylas.utils import timestamp_from_dt, AuthMethod
 
 # pylint: disable=attribute-defined-outside-init
 
@@ -34,6 +35,7 @@ class RestfulModel(dict):
     datetime_filter_attrs = {}
     typed_dict_attrs = {}
     read_only_attrs = {}
+    auth_method = AuthMethod.BEARER
     # The Nylas API holds most objects for an account directly under '/',
     # but some of them are under '/a' (mostly the account-management
     # and billing code). api_root is a tiny metaprogramming hack to let
@@ -54,11 +56,14 @@ class RestfulModel(dict):
     def create(cls, api, **kwargs):
         object_type = kwargs.get("object")
         cls_object_type = getattr(cls, "object_type", cls.__name__.lower())
+        # These are classes that should bypass the check below because they
+        # often represent other types (e.g. a delta's object type might be event)
+        class_check_whitelist = ["jobstatus", "delta"]
         if (
             object_type
             and object_type != cls_object_type
             and object_type != "account"
-            and cls_object_type != "jobstatus"
+            and cls_object_type not in class_check_whitelist
             and not _is_subclass(cls, object_type)
         ):
             # We were given a specific object type and we're trying to
@@ -113,9 +118,11 @@ class RestfulModel(dict):
                 continue
             if hasattr(self, attr):
                 if attr in reserved_keywords:
-                    dct[attr] = getattr(self, "{}_".format(attr))
+                    attr_value = getattr(self, "{}_".format(attr))
                 else:
-                    dct[attr] = getattr(self, attr)
+                    attr_value = getattr(self, attr)
+                if attr_value is not None:
+                    dct[attr] = attr_value
         for date_attr, iso_attr in self.cls.date_attrs.items():
             if date_attr in self.read_only_attrs:
                 continue
@@ -152,17 +159,22 @@ class NylasAPIObject(RestfulModel):
 
     def save(self, **kwargs):
         if self.id:
-            new_obj = self.api._update_resource(
-                self.cls, self.id, self.as_json(), **kwargs
-            )
+            new_obj = self._update_resource(**kwargs)
         else:
-            new_obj = self.api._create_resource(self.cls, self.as_json(), **kwargs)
-        for attr in self.cls.attrs:
-            if hasattr(new_obj, attr):
-                setattr(self, attr, getattr(new_obj, attr))
+            new_obj = self._create_resource(**kwargs)
+        self._update_values(new_obj)
 
     def update(self):
-        new_obj = self.api._update_resource(self.cls, self.id, self.as_json())
+        new_obj = self._update_resource()
+        self._update_values(new_obj)
+
+    def _create_resource(self, **kwargs):
+        return self.api._create_resource(self.cls, self.as_json(), **kwargs)
+
+    def _update_resource(self, **kwargs):
+        return self.api._update_resource(self.cls, self.id, self.as_json(), **kwargs)
+
+    def _update_values(self, new_obj):
         for attr in self.cls.attrs:
             if hasattr(new_obj, attr):
                 setattr(self, attr, getattr(new_obj, attr))
@@ -478,6 +490,7 @@ class Draft(Message):
         "starred",
         "snippet",
         "tracking",
+        "metadata",
     ]
     datetime_attrs = {"last_modified_at": "date"}
     collection_name = "drafts"
@@ -864,7 +877,7 @@ class Component(NylasAPIObject):
     api_root = "component"
 
     def __init__(self, api):
-        NylasAPIObject.__init__(self, RoomResource, api)
+        NylasAPIObject.__init__(self, Component, api)
 
     def as_json(self):
         dct = NylasAPIObject.as_json(self)
@@ -874,6 +887,72 @@ class Component(NylasAPIObject):
         return dct
 
 
+class Webhook(NylasAPIObject):
+    attrs = (
+        "id",
+        "callback_url",
+        "state",
+        "triggers",
+        "application_id",
+        "version",
+    )
+    read_only_attrs = {"id", "application_id", "version"}
+
+    collection_name = "webhooks"
+    api_root = "a"
+
+    def __init__(self, api):
+        NylasAPIObject.__init__(self, Webhook, api)
+
+    def as_json(self):
+        dct = {}
+        # Only 'state' can get updated
+        if self.id:
+            dct["state"] = self.state
+        else:
+            dct = NylasAPIObject.as_json(self)
+        return dct
+
+    class Trigger(str, Enum):
+        """
+        This is an Enum representing all the possible webhook triggers
+
+        see more: https://developer.nylas.com/docs/developer-tools/webhooks/available-webhooks
+        """
+
+        ACCOUNT_CONNECTED = "account.connected"
+        ACCOUNT_RUNNING = "account.running"
+        ACCOUNT_STOPPED = "account.stopped"
+        ACCOUNT_INVALID = "account.invalid"
+        ACCOUNT_SYNC_ERROR = "account.sync_error"
+        MESSAGE_CREATED = "message.created"
+        MESSAGE_OPENED = "message.opened"
+        MESSAGE_UPDATED = "message.updated"
+        MESSAGE_LINK_CLICKED = "message.link_clicked"
+        THREAD_REPLIED = "thread.replied"
+        CONTACT_CREATED = "contact.created"
+        CONTACT_UPDATED = "contact.updated"
+        CONTACT_DELETED = "contact.deleted"
+        CALENDAR_CREATED = "calendar.created"
+        CALENDAR_UPDATED = "calendar.updated"
+        CALENDAR_DELETED = "calendar.deleted"
+        EVENT_CREATED = "event.created"
+        EVENT_UPDATED = "event.updated"
+        EVENT_DELETED = "event.deleted"
+        JOB_SUCCESSFUL = "job.successful"
+        JOB_FAILED = "job.failed"
+
+    class State(str, Enum):
+        """
+        This is an Enum representing all the possible webhook states
+
+        see more: https://developer.nylas.com/docs/developer-tools/webhooks/#enable-and-disable-webhooks
+        """
+
+        ACTIVE = "active"
+        INACTIVE = "inactive"
+
+
 class Namespace(NylasAPIObject):
     attrs = [
         "account",
@@ -905,6 +984,7 @@ class Account(NylasAPIObject):
         "namespace_id",
         "provider",
         "sync_state",
+        "authentication_type",
         "trial",
         "metadata",
     ]
