@@ -16,6 +16,13 @@ def test_search_messages(api_client):
     assert "Pinot" in messages[1].snippet
 
 
+@pytest.mark.usefixtures("mock_message_search_response")
+def test_search_messages_with_limit_offset(mocked_responses, api_client):
+    api_client.messages.search("Pinot", limit=10, offset=0)
+    request = mocked_responses.calls[0].request
+    assert request.path_url == "/messages/search?q=Pinot&limit=10&offset=0"
+
+
 @pytest.mark.usefixtures("mock_message_search_response")
 def test_search_drafts(api_client):
     with pytest.raises(Exception):
